from bossy import loader
from bossy.commands import Command


class Bossfile(object):

    """A Bossfile."""

    NotFound = loader.BossfileNotFound
    LoadingError = loader.BossfileLoadingError

    def __init__(self, bossfile_mod):
        self.mod = bossfile_mod
        self.filename = self.mod.__file__

    def command(self, command_name):
        """Retrieve a :class:`Command` instance, given its name."""

        return self.commands()[command_name]

    def commands(self):
        """Get a dictionary of command names -> :class:`Command` instances."""

        commands = {}
        for name, value in vars(self.mod).iteritems():
            if isinstance(value, Command):
                commands[value.name] = value
        return commands

    def install(self, sub_parsers):

        """
        Install the commands from this Bossfile on a subparser group.

        Returns a dictionary of command names -> command subparsers.
        """

        sub_parser_names = {}
        for name, command in self.commands().iteritems():
            sub_parser_names[name] = command.install(sub_parsers)
        return sub_parser_names

    @classmethod
    def load(cls, location=None):
        """Load a :class:`Bossfile`, optionally with an explicit location."""

        return cls(loader.load_bossfile_module(location=location))



class Command(object):

    """Wraps a Python function and exposes it as a Bossy command."""

    def __init__(self, function, **opts):
        self.function = function
        self.name = opts.pop('name', function.__name__)
        self.opts = opts

        doc = getattr(function, '__doc__', '') or ''
        self.opts.setdefault('description', doc)
        self.opts.setdefault('help', (doc.lstrip('\r\n') or '\n').splitlines()[0])

        self.opts.setdefault('prog', 'bossy %s' % (self.name,))

        self.arguments = getattr(function, 'arguments', [])

    def __call__(self, *args, **kwargs):
        kwargs.pop('__func', None)
        return self.function(*args, **kwargs)

    def add_argument(self, *args, **kwargs):
        self.arguments.append((args, kwargs))

    def install(self, sub_parsers):

        """
        Install this command on an ``argparse`` subparser group.

        Returns the command's subparser.
        """

        sub_parser = sub_parsers.add_parser(self.name,
                                            **self.opts)
        for args, kwargs in self.arguments:
            sub_parser.add_argument(*args, **kwargs)
        sub_parser.set_defaults(__func=self)
        return sub_parser


def command(*args, **kwargs):

    """
    Decorate a function as a command.

    Allows both optionless declaration:

        @command
        def implicit_name():
            pass
        assert isinstance(implicit_name, Command)

    And declaration with options:

        @command(name='explicit_name')
        def function():
            pass
        assert isinstance(function, Command)
        assert function.name == 'explicit_name'
    """

    def make_command_instance(*args):
        return Command(*args, **kwargs)
    if args:
        return make_command_instance(*args)
    return make_command_instance


def argument(*args, **kwargs):
    def decorator(func):
        if not hasattr(func, 'arguments'):
            func.arguments = []
        func.arguments.insert(0, (args, kwargs))
        return func
    return decorator



import errno
import os
import sys
import types


class BossfileLoadingError(Exception):
    """An exception occurred when loading the Bossfile."""

    exc_type = property(lambda self: self.args[0])
    exc_value = property(lambda self: self.args[1])
    exc_traceback = property(lambda self: self.args[2])


class BossfileNotFound(BossfileLoadingError):
    """Couldn't find the Bossfile at the specified location."""
    pass


def load_module(name, filename, install=True):
    """Load a Python module from a filename."""

    mod = types.ModuleType(name)
    mod.__file__ = filename
    execfile(filename, globals(), mod.__dict__)
    if install:
        sys.modules[name] = mod
    return mod


def load_bossfile_module(location=None):

    """
    Load a Bossfile module, with an optional explicit location.

    If no location is provided, a file called ``Bossfile`` in the current
    directory will be used. If the name of a directory is provided, a file
    called ``Bossfile`` in that directory will be used. Otherwise the location
    is treated as the path to the file itself.
    """

    if location is None:
        location = os.path.join(os.getcwdu(), 'Bossfile')
    elif os.path.isdir(location):
        location = os.path.join(location, 'Bossfile')

    try:
        bossfile_mod = load_module('bossfile', location)
    except IOError, exc:
        if exc.errno == errno.ENOENT:
            raise BossfileNotFound(*sys.exc_info())
        raise BossfileLoadingError(*sys.exc_info())
    except Exception, exc:
        raise BossfileLoadingError(*sys.exc_info())

    return bossfile_mod



import argparse
import pdb
import sys
import traceback

import pkg_resources

from bossy.bossfile import Bossfile


def get_version(pkg_name):
    req = pkg_resources.Requirement.parse(pkg_name)
    return pkg_resources.working_set.find(req).version


parser = argparse.ArgumentParser(
    prog='bossy',
    description="Run commands from a Bossfile.",
    usage='bossy [-h|-v|-f BOSSFILE] <command> ...',
    add_help=False)
global_opts = parser.add_argument_group('global options')
global_opts.add_argument('-f', '--file', metavar='BOSSFILE',
                         default='Bossfile',
                         help="Location of an alternative Bossfile to use. "
                         "By default, bossy will look for a file called "
                         "'Bossfile' in the current directory.")
global_opts.add_argument('--pdb', action='store_true', default=False,
                         help="Launch the Python Debugger on uncaught "
                         "exceptions.")
global_opts.add_argument('-v', '--version', action='version',
                         version='bossy ' + get_version('bossy'))


def add_help(parser):
    """Add a help option to a parser afterwards."""

    parser.add_argument('-h', '--help', action='help',
                        default=argparse.SUPPRESS,
                        help='show this help message and exit')


def select_args(parsed_args, subparser):
    selected_args = {}
    for action in subparser._actions:
        if getattr(action, 'dest', None) in parsed_args:
            selected_args[action.dest] = getattr(parsed_args, action.dest)
    return selected_args


def main(args=None):
    pre_args, remaining_args = parser.parse_known_args()
    if pre_args.pdb:
        sys.excepthook = lambda *exc_info: pdb.pm()

    try:
        bossfile = Bossfile.load(pre_args.file)
    except Bossfile.LoadingError, exc:
        add_help(global_opts)
        # This will catch any -h/--help options.
        parser.parse_known_args(remaining_args)

        if isinstance(exc, Bossfile.NotFound):
            parser.error("Couldn't locate the Bossfile in the current directory.")

        error = "Error occurred loading the Bossfile:\n"
        for line in traceback.format_exception(*exc.args):
            error += "  " + line
        parser.error(error)

    subparsers = parser.add_subparsers(title='sub-commands',
                                       metavar='<command>')
    subparser_names = bossfile.install(subparsers)
    add_help(global_opts)
    args = parser.parse_args(remaining_args)
    selected_args = select_args(args, subparser_names[args.__func.name])
    args.__func(**selected_args)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from distribute_setup import use_setuptools
use_setuptools()

from setuptools import setup, find_packages
import os.path as p

VERSION = open(p.join(p.dirname(p.abspath(__file__)), 'VERSION')).read().strip()

setup(
    name='bossy',
    version=VERSION,
    description='Write commands in Python, run them from the shell.',
    author='Zachary Voase',
    author_email='z@zacharyvoase.com',
    url='http://github.com/zacharyvoase/bossy',
    packages=find_packages(where='lib'),
    package_dir={'': 'lib'},
    entry_points={
        'console_scripts': [
            'bossy = bossy.main:main',
        ],
    },
    install_requires=[
        'argparse>=1.2.1',
        'nose>=1.1.2',
    ],
)
